; Starmaster for the Atari 2600 VCS
;
; Copyright 1982 Activision Inc.
; By Alan Miller
;
; Reverse-Engineered by Manuel Polik (101.36834@germanynet.de)
; 23.10.2K      - Started
;
; ??.11.2K      - Thomas Jentzsch sent me his previous attempt to
;                 reverse-engineer Starmaster, which pushes me a great 
;                 step forward!
;
; 09.11.2K      - GameInit Subroutine completed
; 14.11.2K      - Found the frameCounter
; 15.11.2K      - Found the starFieldSpeed
; 15.11.2K      - Found the logoTimer
; 04.12.2K      - Finally understood how horizontal positioning works
; 07.12.2K      - Finished the drawing of the Map
; 11.01.2.1K    - Finished the drawing of the state lines & the logo
;               - Found the enemy coordinates


    include vcs.h

; Equates:

ID_BASE             = 4
ID_SHIP             = 5
Map_WIDTH           = 6
NUM_SECTORS         = 6*6
NUM_BASES           = 4

; RAM variables:

level               = $80   ; Current game variation being played (0-3)
saverColor          = $81   ; 00 for normal, else screen saver
brightnessMask      = $82   ; FF -> full v 7F -> darker saver colors
frameCounter        = $83   ; Incremented every video frame
randomNumber        = $84   ; Random value

; All variables from 85 to B7 are zero'd out when 'Reset' is pressed
notUsed             = $85   ; This is not used for anything useful
logoTimer           = $86   ; 00 in flight, else Activision logo timer

; All variables from 87 to B7 are zero'd out when 'Select' is pressed

saverTimer          = $87   ; If < 127 then in screen saver mode
tempVariable        = $88   ; Various temporary data
tempVariable2       = $89   ; Various temporary data
starFieldSpeed      = $8A   ; 00 = Game, 40 = Intro, 

; The 18 variables from 8B to 9C represent the matrix of the Map screen
; Every byte stores two of the 36 sectors of the Map
; Important note: The High & the Low nibble are swapped!
;
; This is what the Map looks like at Startup:
;
; $9A [CLEAR/CLEAR]   $9B [*TWO*/THREE] $9C [CLEAR/CLEAR]
; $97 [*CLEAR*/BASE]  $98 [CLEAR/CLEAR] $99 [*CLEAR*/BASE]
; $94 [CLEAR/CLEAR]   $95 [*CLEAR*/ONE] $96 [CLEAR/CLEAR]
; $91 [*CLEAR*/CLEAR] $92 [CLEAR/CLEAR] $93 [*CLEAR*/CLEAR]
; $8E [CLEAR/CLEAR]   $8F [*ONE*/CLEAR] $90 [BASE/PLAYER]
; $8B [*TWO*/BASE]    $8C [CLEAR/CLEAR] $8D [*CLEAR*/CLEAR]
;
; Name:     Nibblevalue:    Meaning:
; CLEAR     00              Clear Sector
; ONE       01              1 enemy in Sector
; TWO       02              2 enemies in Sector
; THREE     03              3 enemies in Sector
; BASE      04              Base in Sector
; PLAYER    05              Player in Sector 
; * = sectors ORed with the current level, bringing 0-3 enemies in!

sectorList          = $8B   ; $8B-$9C

tempVariable3       = $A4   ; Various temporary data

flashTimerBlue      = $A7   ; Timer for blue flash
flashTimerRed       = $A8   ; Timer for red flash
flashTimerOrange    = $A9   ; Timer for orange flash

enemy0Y             = $AE   ; Vertical position of enemy 0 object
enemy1Y             = $AF   ; Vertical position of enemy 1 object
enemy0X             = $B0   ; Horizontal position of enemy 0 object
enemy1X             = $B1   ; Horizontal position of enemy 1 object
enemy0Z             = $B2   ; Distance of enemy 0 object
enemy1Z             = $B3   ; Distance of enemy 1 object

tuneIndex           = $B6

; All variables from B8 to E0 are reseted with ROM values every new game
; (Values from inittab-FD0A)

levelDelay          = $B8   ; countdown until level is skipped
damageLS            = $B9   ; Repaired (A) / Laser (C) / Shield(B)
damageWR            = $BA   ; Repaired (A) / Radar(D) / Warp (E)
bcdenergyHi         = $BB   ; Hibyte of the energy
bcdenergyLo         = $BC   ; Lowbyte of the enegy
bcdstarDateHi       = $BD   ; Hibyte of the star date
bcdstarDateLo       = $BE   ; Lowbyte of the star date
bcdwarpEnergyHi     = $BF   ; Hibyte of the warp energy
bcdwarpEnergyLo     = $C0   ; Lowbyte of the star date
bcdscoreHi          = $C1   ; Hibyte of the score
bcdscoreLo          = $c2   ; Lowbyte of the score
shipSector          = $C3   ; current ship sector
targetSector        = $C4   ; desired ship sector

loopCounter         = $E8   ; Additional counter, when X/Y aren't enough

vertShipSize        = $E9   ; verticl size of the enemy ship in pixel
vertShotSize        = $EA   ; verticl size of the enemy shot in pixel

enemyShipShape      = $EB   ; Pointer to current enemy ship shape
;enemyShotShape($EC), current enemy shot shape, not directly accessed

; Next 12 Bytes are Spritepointers

targetPtr           = $ED   ; This points to base v enemy graphics
Sprite01Pointer     = $EF   ; Sprite 01 Pointer / 2 Bytes
Sprite02Pointer     = $F1   ; Sprite 02 Pointer / 2 Bytes
Sprite03Pointer     = $F3   ; Sprite 03 Pointer / 2 Bytes
Sprite04Pointer     = $F5   ; Sprite 04 Pointer / 2 Bytes
Sprite05Pointer     = $F7   ; Sprite 05 Pointer / 2 Bytes

; mapCombatState stores the information if Map is displayed or not in
; the highest bit. Other Bits hold states of B/W & difficulty switches

mapCombatState      = $F9   ; Map DifP0 DifP1 . . ColBW . .
bcdEnemyCounter     = $FA   ; Number of enemy ships as BCD.
LevelSprite         = $FB   ; Pointer to sprite representing game number

; Begin source

    processor 6502
    ORG $F000
    
; Reset everything

Start:              SEI                 ; Disable Interrupts
                    CLD                 ; Binary mode
                    LDX #$FF            ;
                    TXS                 ; Init Stack
                    INX                 ;
                    JSR GameInit        ; 
                    LDA #$40            ;
                    STA starFieldSpeed  ; Fast starfield in intro
                    STA logoTimer       ; Logo animation started
                    LDA #$12            ;
                    STA tuneIndex       ; Play win tune in intro

; Set background, playfield & ball colors 
; and flash in the right colors, if necessary

Mainloop:           LDX #$01            ; Set background color black
                    LDY #$01            ; 2 color values to write
                    LDA frameCounter    ; Flash 4 frames / 
                    AND #$04            ;     Then don't flash 4 frames
                    BEQ SetBackColors   ; Y: SetBackColors (X=01)
                    TAX                 ; N: X -> 04
CheckFlashSrc:      LDA $A5,X           ; Flashtimer (A9-A7) > 00 ?
                    BNE SetBackColors   ; Y: SetBackCol. X= 04 v 03 v 02
                    DEX                 ; N: Continue checking
                    CPX #$01            ; All flash sources checked?
                    BNE CheckFlashSrc   ; N: Check next
SetBackColors:      LDA backcoltab,X    ; Y: Load color...
                    EOR saverColor      ; ...change it...
                    AND brightnessMask  ; ...when in saver mode
                    STA.wy $0008,Y      ; Write to both COLUPF & COLUBK   
                    DEX                 ; Next color in table
                    DEY                 ; Second color register
                    BPL SetBackColors   ; Write second color

; Set the Target Pointer to the right memory page
; FF for friendly targets, FE for hostile targets
; A small base actually is a comet, the shape is switched when
; it's coming closer

                    LDA starFieldSpeed  ; Warping?
                    BNE EnemyAsTarget   ; Y: Show enemy as target
                    JSR ScanShipSector  ; N: Scan ship sector
                    TAX                 ; X-> Scan result
                    CMP #ID_BASE+ID_SHIP; Base near?
                    BNE EnemyAsTarget   ; N: Show enemy as target
                    LDA enemy0Z         ; A -> distance to base
                    CMP #$50            ; Base already close enough?
                    BCC BaseAsTarget    ; Y: Show base as target
EnemyAsTarget:      DEY                 ; N: Y -> FE
BaseAsTarget:       STY targetPtr+1     ; MSB of targetPtr -> FF v FE

; Set the colors of both players and missiles
; blue if in a base or clear sector
; Yellow if warping
; *glowing* if enemies near

                    LDA #$2C            ; Yellow (for comets)
                    LDY starFieldSpeed  ; Warping?
                    BNE SetTargetColors ; Y: Yellow players / missiles
                    LDA #$8C            ; N: Blue (base & clear sectors)
                    CPX #ID_BASE+ID_SHIP; Base Near? (X -> Scan result!)
                    BEQ SetTargetColors ; Y: Blue players / missiles
                    CPX #ID_SHIP        ; N: Sector clear?
                    BEQ SetTargetColors ; Y: Blue players / missiles
                    LDA frameCounter    ; N: Colorcycle (glowing target)
SetTargetColors:    EOR saverColor      ; Change Color when
                    AND brightnessMask  ; in saver mode
                    STA COLUP0          ; Store color (same color for...
                    STA COLUP1          ; ...target, crosshair & enemy!)

; Check console switches and swap between Map/Combat view if B/W, or any
; of the difficulty switches is pressed _and_ we're not warping

                    LDA logoTimer       ; Logo Drawn?
                    BNE checkSwitches   ; N: We are not in flight
                    LDA starFieldSpeed  ; Y: Warping?
                    BNE IgnoreSwitches  ; Y: Ignore Switches
checkSwitches:      LDA SWCHB           ; N: Check Switches
                    LSR                 ;
                    AND #$64            ; A-> .DifP0DifP1. .ColBW..
                    STA tempVariable    ; Store temporary
                    EOR mapCombatState  ; EOR New with previous values
                    AND #$64            ; Switch values differ?
                    BEQ IgnoreSwitches  ; N: change Nothing
                    LDA mapCombatState  ; Y: Load previous values
                    EOR #$80            ; Swap between map & combat
                    AND #$80            ; Mask map/combat bit
                    ORA tempVariable    ; Set new switch values
                    STA mapCombatState  ; Save new values
                    LDA targetSector    ; Since it possibly switched...
                    TAY                 ; ...from combat to map,...
                    JSR RemoveShip      ; ...clear old target pos. ...
                    LDA shipSector      ; ...and reset it with...
                    STA targetSector    ; ...the current ship position
IgnoreSwitches:     LDA mapCombatState  ; Map?
                    BPL ShowCombat      ; N: Draw combat screen
                    JMP ShowMap         ; Y: Draw map screen

ShowCombat:         LDX #$8F            ; 
                    STX REFP1           ; Reflect player 1
                    TXS                 ; S -> $8F
                    LDX #$00            ; X -> $00

; Select enemy shape we have to draw first.

                    LDA enemy0Z         ; A -> distance to current enemy
                    LSR                 ; Divide by 32 
                    LSR                 ; 
                    LSR                 ;
                    LSR                 ;
                    LSR                 ; A -> '00000xxx' 
                    ORA #$F8            ; A -> '11111xxx'
                    SEC                 ; A -> values from -1 to -8
                    ADC enemy0Y         ; A += enemy0Y
                    CMP #$8E            ; enemy ship out of sight?
                    BCS ShowEnemyShot   ; Y: Show the shot
                    CMP enemy1Y         ; N: enemy shot below ship?
                    BCS ShowEnemyship   ; Y: Show enemy ship
                    LDA enemy1Y         ;
                    CMP #$8E            ; enemy shot out of sight?
                    BCS ShowEnemyship   ; Y: Show the ship
ShowEnemyShot:      INX                 ; Switch to shot shape
ShowEnemyship:      LDA enemyShipShape,X; load current shape...
                    STA targetPtr       ; and store it in spritepointer

; This prepares everything for a second enemy object

                    LDY vertShipSize,X  ; Y -> size of sprite to draw
                    STY tempVariable2   ; store temporary
                    LDY #$70            ; Assume Y -> $70
                    SEC                 ;
                    LDA enemy0Y,X       ; A -> y pos of sprite to draw
                    SBC #$91            ; y pos < $91?
                    BCS acceptYPos      ; Y: Accept Y pos
                    TAY                 ; Y -> y pos - $91
acceptYPos:         TXA                 ; A -> enemy obj to draw first
                    EOR #$01            ; swap to second object
                    TAX                 ; X-> second object
                    LDA enemy0Y,X       ; A -> Y pos of second object
                    STA tempVariable3   ; stored temporary
                    LDA enemyShipShape,X; A -> shape of next object
                    STA enemyShipShape  ; store as ship shape
                    LDA vertShipSize,X  ; A -> size of nect object
                    STA vertShipSize    ; store as ship size
                    LDA enemy0X         ; ship is horiz. in sight...
                    CMP #$09            ; ...between $09 & $A2?
                    BCC shipInSight     ;
                    CMP #$A2            ;
                    BCC shipNotInSight  ;
shipInSight:        LDY #$70            ; Y: Y->70
shipNotInSight:     LDX #$07            ; N: Shot is...
                    STX enemy1X         ; ...invisible, too

ShowMap:            LDA INTIM           ;
                    BPL ShowMap         ; Wait until VBLANK is done
                    LDX #$04            ;
WasteLines:         STA WSYNC           ;
                    DEX                 ;
                    BNE WasteLines      ; Wait 4 more lines
                    STX VBLANK          ; Stop vertical blank


WaitVBlank:         LDA INTIM           ;
                    BPL WaitVBlank      ; Wait until VBLANK is done
                    LDA #$00            ; A -> $00
                    STA WSYNC           ; Finish current line
                    STA HMOVE           ; Move sprites horizontally
                    STA VBLANK          ; Stop vertical blank
                    BEQ LF150           ;2
LF0F8:              BCS LF100           ;2
                    LDA (targetPtr),Y   ;5
                    STA GRP0            ;3
                    STA GRP1            ;3
LF100:              LDA #$02            ;2
                    STA ENABL           ;3
                    TSX                 ;2
                    DEX                 ;2
                    LDA #$00            ;2
                    CPX $AD             ;3
                    BCC LF112           ;2
                    CPX $AC             ;3
                    BCS LF116           ;2
                    LDA #$02            ;2
LF112:              STA ENAM0           ;3
                    STA ENAM1           ;3
LF116:              STA WSYNC           ;3
                    STA HMOVE           ;3
                    DEX                 ;2
                    INY                 ;2
                    CPY #$3C            ;2
       BCS    LF126   ;2
       LDA    (targetPtr),Y ;5
       STA    GRP0    ;3
       STA    GRP1    ;3
LF126: DEX            ;2
       LDA    #$00    ;2
       INY            ;2
       CPY    #$3C    ;2
       BCS    LF130   ;2
       LDA    (targetPtr),Y ;5
LF130: STA    WSYNC   ;3
       STA    HMOVE   ;3
       STA    GRP0    ;3
       STA    GRP1    ;3
       LDA    #$00    ;2
       STA    ENABL   ;3
       CPX    $AD     ;3
       BCC    LF146   ;2
       CPX    $AC     ;3
       BCS    LF14A   ;2
       LDA    #$02    ;2
LF146: STA    ENAM0   ;3
       STA    ENAM1   ;3
LF14A: DEX            ;2
       TXS            ;2
       DEC    enemy1X     ;5
       LDX    enemy1X     ;3
LF150: LDA    $E0,X   ;4
       STA    HMBL    ;3
       AND    #$0F    ;2
       STA    tempVariable     ;3
       INY            ;2
       LDA    (targetPtr),Y ;5
       CPY    #$3C    ;2
       STA    WSYNC   ;3
       STA    HMOVE   ;3
       BCS    LF169   ;2
       STA    GRP0    ;3
       STA    GRP1    ;3
       BCC    LF16D   ;2
LF169: NOP            ;2
       NOP            ;2
       NOP            ;2
       NOP            ;2
LF16D: NOP            ;2
       LDA    VSYNC   ;3
       LDX    tempVariable     ;3
LF172: DEX            ;2
       BPL    LF172   ;2
       STA    RESBL   ;3
LF177: TSX            ;2
       DEX            ;2
       INY            ;2
       CPY    #$3C    ;2
       STA    WSYNC   ;3
       STA    HMOVE   ;3
       BCS    LF188   ;2
       LDA    (targetPtr),Y ;5
       STA    GRP0    ;3
       STA    GRP1    ;3
LF188: TXA            ;2
       AND    #$01    ;2
       BEQ    LF19D   ;2
       CPX    $AD     ;3
       BCC    LF196   ;2
       CPX    $AC     ;3
       BCS    LF196   ;2
       ASL            ;2
LF196: STA    ENAM0   ;3
       STA    ENAM1   ;3
       JMP    LF1A5   ;3
LF19D: CPY    #$3C    ;2
       BCS    LF1A5   ;2
       CPY    tempVariable2     ;3
       BCS    LF1BD   ;2
LF1A5: TXS            ;2
       TXA            ;2
       LDX    enemy1X     ;3
       STX    HMBL    ;3
       CMP    $CA,X   ;4
       BCS    LF177   ;2
       CMP    #$06    ;2
       BCC    LF1EC   ;2
       INY            ;2
       CPY    #$3C    ;2
       STA    WSYNC   ;3
       STA    HMOVE   ;3
       JMP    LF0F8   ;3
LF1BD: LDY    vertShipSize     ;3
       STY    tempVariable2     ;3
       LDY    #$70    ;2
       SEC            ;2
       LDA    tempVariable3     ;3
       STX    tempVariable     ;3
       SBC    tempVariable     ;3
       BPL    LF1CD   ;2
       TAY            ;2
LF1CD: LDA    enemyShipShape     ;3
       STA    targetPtr     ;3
       TXS            ;2
       TXA            ;2
       LDX    enemy1X     ;3
       STX    HMBL    ;3
       STA    WSYNC   ;3
       STA    HMOVE   ;3
       CMP    $CA,X   ;4
       BCS    LF1E6   ;2
       CMP    #$06    ;2
       BCC    LF1EA   ;2
       JMP    LF100   ;3
LF1E6: TSX            ;2
       DEX            ;2
       BNE    LF188   ;2
LF1EA: SBC    #$00    ;2
LF1EC: TAX            ;2
LF1ED: STA    WSYNC   ;3
       STA    HMOVE   ; (3)
       LDA    #$00    ; (5)
       STA    GRP0    ; (8)
       STA    GRP1    ; (11)
       DEX            ; (13)
       BPL    LF1ED   ; (15)
       TXS            ; (17)
       JSR    PrepareStates   ;6
       LDX    #$01    ; 2 lines of states in combat mode
       STX    REFP1   ;3
       BNE    ShowStates   ;2

ShowMap:            LDA INTIM           ;
                    BPL ShowMap         ; Wait until VBLANK is done
                    LDX #$04            ;
WasteLines:         STA WSYNC           ;
                    DEX                 ;
                    BNE WasteLines      ; Wait 4 more lines
                    STX VBLANK          ; Stop vertical blank

; Set both player sprites

                    LDA #$2E            ; Horizontal X Position = 37(d)
                    JSR SetXPos         ; Set Player 0 Horizontal
                    INX                 ; Switch to player 1
                    LDA #$5E            ; Horizontal X Position = 85(d)
                    JSR SetXPos         ; Set Player 1 Horizontal
                    LDY #$03            ; players 3 copies/close spacing
                    STY NUSIZ0          ;
                    STY NUSIZ1          ;
                    STA WSYNC           ; Finish current line
                    STA HMOVE           ; Fine positioning during HBLANK

; Change MSBs of sprite poiters to page FF
                                        
                    LDX #$0A            ; Set 6 MSBs of sprite pointers
                    LDA #$FF            ; ... to FF in Map mode
InitSpritePtrs:     STA $EE,X           ; Write MSB
                    DEX                 ; Skip to next pointer
                    DEX                 ; Done?
                    BPL InitSpritePtrs  ; N: Continue

; Set sprite colors

                    LDA #$2C            ; Y: Map Sprites are yellow...
                    EOR saverColor      ; ...or darker...
                    AND brightnessMask  ; ...when in saver mode...
                    STA COLUP0          ; 
                    STA COLUP1          ; Store color

; Draw all 36 Map sectors

                    STA HMCLR           ; Clear h-mot registers
                    STA WSYNC           ; Finish current line
                    LDX #$23            ; 36 (d) sectors to draw
                    STX loopCounter     ; Store in loopCounter
DoNextMapline:      LDY #$0A            ; 6 (d) sectors per line 
SetMapSpritePtrs:   LDA loopCounter     ; Load sector number
                    JSR ScanSector      ; Scan it
                    TAX                 ; X -> Scanresult
                    LDA damageWR        ; Load damageWR
                    AND #$0F            ; Mask radarstate
                    CMP #$0D            ; Damaged?
                    BNE RadarOK         ; N: RadarOK
                    LDA noradartab,X    ; Y: show only bases and ship
                    TAX                 ;
RadarOK:            LDA mapshapeptrtab,X; Load offset to mapsprite
                    STA.wy $00ED,Y      ; Set all sprite pointer LSBs
                    DEC loopCounter     ; next sector
                    DEY                 ; next sprite pointer
                    DEY                 ; Finished?
                    BPL SetMapSpritePtrs; N: Continue
                    JSR DrawMapLine     ; Draw mapline
                    LDA loopCounter     ; All sectors drwan?
                    BPL DoNextMapline   ; N: Do next mapline

; Draw space between Map and statelines

                    LDX #$18            ; 25 lines space before states
DrawMapSpace:       STA WSYNC           ; Finish current line
                    LDA VSYNC           ; (3)These are all dummy...
                    NOP                 ; (5)instructions. So we call
                    NOP                 ; (7)PrepareStates at cycle 17!
                    NOP                 ; (9) 
                    NOP                 ; (11)
                    DEX                 ; (13)
                    BPL DrawMapSpace    ; (15)
                    NOP                 ; (17)
                    JSR PrepareStates   ; -> After 25 lines + 17 cycles

; Draw the stateslines. 4 in Map- and 1 in Combatmode

                    LDX #$04            ; 5 lines of states in map mode
ShowStates:         STX loopCounter     ; Draw 5 v 2 statelines

; Set sprite colors

                    LDA #$2C            ; Y: State chars are yellow...
                    EOR saverColor      ; ...or darker...
                    AND brightnessMask  ; ...when in saver mode...
                    STA COLUP0          ; 
                    STA COLUP1          ; Store color

; Set sprite pointers & draw statelines
                   
                    LDA #$B1            ; Set second sprite-pointer...
                    STA Sprite01Pointer ; ... to the ':' shape
NextStateLine:      LDA LevelSprite     ; A-> Offset to level shape
                    CPX #$04            ; Did we start in Map Mode?
                    BEQ ComingFromMap   ; Y: Use level shape first
                    LDA stateptrtab,X   ; N: A-> 'D' v 'E' v 'S' v 'W'
ComingFromMap:      STA targetPtr       ; Set first sprite-pointer
                    TXA                 ; A -> 0 v 1 v 2 v 3 v 4
                    ASL                 ; A -> 0 v 2 v 4 v 6 v 8
                    TAX                 ; X -> 0 v 2 v 4 v 6 v 8
                    INX                 ; select low values
                    TXS                 ; X -> stackpointer (oh, nasty!)
                    LDY #$04            ; Sprite pointers 5&6 first
HigherPointers:     LDA damageLS,X      ; load low/high values of states
                    LSR                 ; select lower nibble... 
                    LSR                 ; ...of status value (bcd!)
                    LSR                 ; (numbers 0-9, damages A-E)
                    LSR                 ;
                    TAX                 ; X-> shape of value
                    LDA stateshapeptr,X ; Load pointer to desired value
                    STA.wy $00F1,Y      ; Store 5th(3rd) sprite pointer
                    TSX                 ; X -> 1 v 3 v 5 v 7 v 9
                    LDA damageLS,X      ; load Low values of statelines
                    AND #$0F            ; select higher nibble
                    TAX                 ; X-> shape of value
                    LDA stateshapeptr,X ; Load pointer to desired value
                    STA.wy $00F3,Y      ; Store 6th(4th) sprite pointer
                    TSX                 ; X -> 1 v 3 v 5 v 7 v 9
                    DEX                 ; select high values
                    TXS                 ; X -> stackpointer (... nasty!)
                    DEY                 ; skip to sprite-pointers 3 & 4
                    DEY                 ;
                    DEY                 ;
                    DEY                 ;
                    BPL HigherPointers  ; Write higher pointers, too
                    LDX #$FF            ; Restore stack pointer after...
                    TXS                 ; ...rude missuse
                    JSR DrawStateLine   ; Draw State Line
                    DEC loopCounter     ; One stateline drawn
                    LDX loopCounter     ; All lines finished?
                    BPL NextStateLine   ; N: draw next

; Draw the Activision Logo

                    STA WSYNC           ; Y: Finished Current line
                    LDA #$00            ; 
                    STA PF2             ; Clear Playfield
                    LDX #$0B            ; Let Sprite pointers... 
PointToLogo:        LDA logoptrtab,X    ; ...point to the Activision...
                    STA targetPtr,X     ; ...logo
                    DEX                 ; Done?
                    BPL PointToLogo     ; N: Continue
                    JSR DrawLogo        ; Draw the logo

; Handle the logo timer. The logo timer is either held always at '00',
; (in flight) or permanently decreased being always nonzero. 
; (-> It jumps from from $01 to $FF!)

                    LDA logoTimer       ; logoTimer = 00?
                    BEQ LTDecreaseDone  ; Y: leave unchanged
                    DEC logoTimer       ; N: Decrease timer
                    BNE LTDecreaseDone  ; Reached '00'?
                    DEC logoTimer       ; Y: Decrease once more

; Check Console Switches

LTDecreaseDone:     LDY #$19            ; 
                    STY TIM64T          ; Set timer for Overscan
                    LDA SWCHB           ; Load console switches
                    LSR                 ; Reset pressed?
                    BCS CheckSelect     ; N: Check Select
                    LDX #$85            ; Y: X->$85
FinishSelect:       JSR GameInit        ; Reset variables
                    JMP EndOverscan     ; End Overscan 
CheckSelect:        LSR                 ; Select pressed?
                    BCS StartLevelDelay ; N: Do something
                    LDA levelDelay      ; Y: Leveldelay = 0?
                    BEQ IncreaseLevel   ; Y: Increase level
                    DEC levelDelay      ; Restart Leveldelay
                    BPL SwitchesDone    ; Just BPL instead of JMPing
IncreaseLevel:      LDA level           ; A -> current level number
                    CLC                 ;
                    ADC #$01            ; increment level number
                    AND #$03            ; mask '000000XX'
                    STA level           ; skip to next level (0-3)
                    LDX #$87            ; X ->$87
                    STX notUsed         ; ????? Useless instruction
                    INC logoTimer       ; Timer +=1?
                    BNE FinishSelect    ; Just BNE instead of JMPing
StartLevelDelay:    STX levelDelay      ; Start level delay with 85 v 87

; Overscan calculations...

SwitchesDone:       JSR NewRandomValue  ; Create random value each frame
                    LDA logoTimer       ; Are we flying?
                    BEQ FlightOverscan  ; Y: do in-flight calculations
                    JMP CommonOverscan  ; N: do common overscan calcs.
FlightOverscan:     LDA frameCounter    ;3
                    BNE LF338           ;2
                    SED                 ;2
                    CLC                 ;2
                    LDA $BE             ;3
                    ADC #$01            ;2
                    STA $BE             ;3
                    LDA $BD             ;3
                    ADC #$00            ;2
                    STA $BD             ;3
                    CLD                 ;2
LF338:              DEC $DD             ;5
                    BEQ LF33F           ;2
                    JMP LF419           ;3
LF33F:              LDA #$60            ;2
                    STA $DD             ;3
                    LDX $AB             ;3
                    LDA LFC67,X         ;4
                    JSR ScanSector      ;6
                    CMP #$04            ;2
                    BEQ LF35C           ;2
                    CMP #$09            ;2
       BEQ    LF35C   ;2
       LDX    $AB     ;3
       CPX    #$03    ;2
       BEQ    LF35A   ;2
       INX            ;2
LF35A: STX    $AB     ;3
LF35C: DEC    $DE     ;5
       BPL    LF364   ;2
       LDA    #$23    ;2
       STA    $DE     ;3
LF364: LDX    $AB     ;3
       LDA    LFC39,X ;4
       LDX    #$05    ;2
LF36B: LDY    LFC95,X ;4
       CPY    $DE     ;3
       BNE    LF374   ;2
       AND    #$0B    ;2
LF374: LDY    LFC9B,X ;4
       CPY    $DE     ;3
       BNE    LF37D   ;2
       AND    #$07    ;2
LF37D: DEX            ;2
       BPL    LF36B   ;2
       STA    $E1     ;3
       TAX            ;2
       LDA    #$0F    ;2
       STA    $E4     ;3
       LDA    $DE     ;3
       CLC            ;2
       ADC    LFD09,X ;4
       BPL    LF395   ;2
       LDA    #$0D    ;2
       STA    $E4     ;3
       BNE    LF39B   ;2
LF395: CMP    #$24    ;2
       BCC    LF39B   ;2
       DEC    $E4     ;5
LF39B: LDA    $E1     ;3
       AND    $E4     ;3
       STA    $E1     ;3
       LDA    $DE     ;3
       JSR    ScanSector   ;6
       CMP    #$04    ;2
       BCS    LF3C9   ;2
       STA    $E2     ;3
       LDA    $DE     ;3
       CLC            ;2
       LDX    $E1     ;3
       ADC    LFD09,X ;4
       STA    $E3     ;3
       JSR    ScanSector   ;6
       BNE    LF3C9   ;2
       LDA    $DE     ;3
       LDY    #$00    ;2
       JSR    LFB87   ;6
       LDA    $E3     ;3
       LDY    $E2     ;3
       JSR    LFB87   ;6
LF3C9: LDX    $DB     ;3
       LDA    LFC67,X ;4
       JSR    ScanSector   ;6
       STA    $E4     ;3
       CMP    #$04    ;2
       BEQ    LF3DB   ;2
       CMP    #$09    ;2
       BNE    LF411   ;2
LF3DB: LDA    $DB     ;3
       ASL            ;2
       ASL            ;2
       ASL            ;2
       CLC            ;2
       ADC    $DC     ;3
       TAX            ;2
       LDA    LFC6B,X ;4
       JSR    ScanSector   ;6
       TAY            ;2
       LDX    $DB     ;3
       LDA    $C5,X   ;4
       SEC            ;2
       SBC    LFC40,Y ;4
       STA    $C5,X   ;4
       BCS    LF409   ;2
       DEC    $C9     ;5
       LDA    $E4     ;3
       SEC            ;2
       SBC    #$04    ;2
       TAY            ;2
       LDA    LFC67,X ;4
       JSR    LFB87   ;6
       LDA    #$B0    ;2
       STA    flashTimerOrange     ;3
LF409: DEC    $DC     ;5
       BPL    LF419   ;2
       LDA    #$07    ;2
       STA    $DC     ;3
LF411: DEC    $DB     ;5
       BPL    LF419   ;2
       LDA    #$03    ;2
       STA    $DB     ;3
LF419: LDA    frameCounter     ;
       AND    #$03    ; Every 4th frame
       BNE    LF425   ; Do this:
       LDA    starFieldSpeed     ; Decrement starFieldSpeed
       BEQ    LF425   ; if it's not
       DEC    starFieldSpeed     ; already zero
LF425: LDA    $AA     ;3
       BEQ    LF42D   ;2
       DEC    $AA     ;5
       BNE    LF457   ;2
LF42D: LDA    #$4E    ;2
       STA    $AC     ;3
       LDA    #$45    ;2
       STA    $AD     ;3
       LDA    damageLS     ;3
       ORA    $A0     ;3
       CMP    #$B0    ;2
       BCS    CommonOverscan   ;2
       LDA    starFieldSpeed     ;3
       CMP    #$C0    ;2
       BCS    CommonOverscan   ;2
       LDA    REFP1   ;3
       ORA    mapCombatState     ;3
       BMI    CommonOverscan   ;2
       LDA    #$20    ;2
       STA    $AC     ;3
       LDA    #$06    ;2
       STA    $AD     ;3
       STX    CXCLR   ;3
       LDX    #$17    ;2
       STX    $AA     ;3
LF457: LDA    $AD     ;3
       CLC            ;2
       ADC    #$03    ;2
       STA    $AD     ;3
       LDA    $AC     ;3
       CLC            ;2
       ADC    #$02    ;2
       CMP    #$4C    ;2
       BCS    CommonOverscan   ;2
       STA    $AC     ;3
CommonOverscan: LDA    $AA     ;3
       LSR            ;2
       LSR            ;2
       LSR            ;2
       TAX            ;2
       LDA    LFF00,X ;4
       STA    NUSIZ0  ;3
       STA    NUSIZ1  ;3
       LDA    SWCHA   ;4
       LSR            ;2
       LSR            ;2
       LSR            ;2
       LSR            ;2
       EOR    #$0F    ;2
       TAY            ;2
       BEQ    LF484   ;2
       STA    saverTimer     ;3
LF484: LDA    logoTimer     ;3
       ORA    $A0     ;3
       BEQ    LF48C   ;2
       LDY    #$00    ;2
LF48C: STY    tempVariable     ;3
       LDA    $A0     ;3
       BNE    LF4D5   ;2
       DEC    $B5     ;5
       BPL    LF4D5   ;2
       LDA    starFieldSpeed     ;3
       BMI    LF49F   ;2
       EOR    #$FF    ;2
       SEC            ;2
       SBC    #$68    ;2
LF49F: LSR            ;2
       LSR            ;2
       LSR            ;2
       SEC            ;2
       SBC    #$10    ;2
       STA    $B5     ;3
       LDX    #$07    ;2
LF4A9: LDA    $CA,X   ;4
       LSR            ;2
       LSR            ;2
       LSR            ;2
       LDY    tempVariable     ;3
       CLC            ;2
       ADC    LFC8B,Y ;4
       TAY            ;2
       LDA    LFD80,Y ;4
       CLC            ;2
       ADC    $CA,X   ;4
       STA    $CA,X   ;4
       LDA    $D2,X   ;4
       LSR            ;2
       LSR            ;2
       LSR            ;2
       LSR            ;2
       TAY            ;2
       LDA    LFCB6,Y ;4
       LDY    tempVariable     ;3
       CLC            ;2
       ADC    LFEE5,Y ;4
       CLC            ;2
       ADC    $D2,X   ;4
       STA    $D2,X   ;4
       DEX            ;2
       BPL    LF4A9   ;2
LF4D5: LDX    #$01    ;2
       LDY    tempVariable     ;3
LF4D9: LDA    enemy0Y,X   ;4
       CLC            ;2
       ADC    LFD25,Y ;4
       STA    enemy0Y,X   ;4
       DEX            ;2
       BPL    LF4D9   ;2
       LDA    enemy0X     ;3
       CLC            ;2
       ADC    LFF10,Y ;4
       STA    enemy0X     ;3
EndOverscan: LDY    INTIM   ;4
       BPL    EndOverscan   ;2
       INC    frameCounter     ;5
       BNE    LF4FC   ;2
       INC    saverTimer     ;5
       BNE    LF4FC   ;2
       SEC            ;2
       ROR    saverTimer     ;5
LF4FC: LDX    #$03    ;2
       STX    WSYNC   ;3
       STX    VSYNC   ;3
       STX    VBLANK  ;3
       LDY    #$FF    ;2
       LDA    #$00    ;2
       BIT    saverTimer     ;3
       BPL    LF511   ;2
       LDY    #$F7    ;2
       LDA    saverTimer     ;3
       ASL            ;2
LF511: STY    brightnessMask     ;3
       STA    saverColor     ;3
       STA    WSYNC   ;3
       LDY    #$40    ;2
       STY    VSYNC   ;3
       STY    TIM64T  ;4
       LDA    $AA     ;3
       CMP    #$16    ;2
       BNE    LF529   ;2
       LDA    #$01    ;2
       JSR    LFBCB   ;6
LF529: LDX    #$07    ;2
LF52B: LDA    $D2,X   ;4
       CMP    #$14    ;2
       BCC    LF545   ;2
       CMP    #tempVariable    ;2
       BCS    LF545   ;2
       LDA    $CA,X   ;4
       CMP    #starFieldSpeed    ;2
       BCS    LF545   ;2
       CMP    #$07    ;2
       BCC    LF545   ;2
LF53F: DEX            ;2
       BPL    LF52B   ;2
       JMP    LF5D4   ;3
LF545: STX    tempVariable     ;3
       LDX    #$07    ;2
LF549: LDA    $CA,X   ;4
       CMP    #$48    ;2
       BCC    LF552   ;2
       DEX            ;2
       BPL    LF549   ;2
LF552: INX            ;2
       STX    tempVariable2     ;3
       LDX    tempVariable     ;3
       CPX    tempVariable2     ;3
       BCC    LF5A8   ;2
LF55B: DEX            ;2
       BMI    LF56C   ;2
       CPX    tempVariable2     ;3
       BCC    LF56C   ;2
       LDA    $CA,X   ;4
       STA    $CB,X   ;4
       LDA    $D2,X   ;4
       STA    $D3,X   ;4
       BCS    LF55B   ;2
LF56C: LDX    tempVariable2     ;3
       LDA    #$53    ;2
       STA    $CA,X   ;4
       CPX    #$04    ;2
       BCS    LF57E   ;2
       LDA    #$3F    ;2
       STA    $CA,X   ;4
       DEX            ;2
       JMP    LF5BA   ;3
LF57E: CLC            ;2
       ADC    #$06    ;2
       CMP    $CB,X   ;4
       BCC    LF597   ;2
       LDA    $D3,X   ;4
       STA    $D2,X   ;4
       LDA    $CB,X   ;4
       STA    $CA,X   ;4
       CLC            ;2
       ADC    #$06    ;2
       STA    $CB,X   ;4
       INX            ;2
       CPX    #$07    ;2
       BCC    LF57E   ;2
LF597: JSR    NewRandomValue   ;6
       LDA    randomNumber     ;3
       AND    #$3F    ;2
       CLC            ;2
       ADC    #$30    ;2
       STA    $D2,X   ;4
       LDX    tempVariable     ;3
       JMP    LF53F   ;3
LF5A8: DEX            ;2
       DEC    tempVariable2     ;5
LF5AB: INX            ;2
       CPX    tempVariable2     ;3
       BCS    LF56C   ;2
       LDA    $CB,X   ;4
       STA    $CA,X   ;4
       LDA    $D3,X   ;4
       STA    $D2,X   ;4
       BCC    LF5AB   ;2
LF5BA: SEC            ;2
       SBC    #$06    ;2
       CMP    $CA,X   ;4
       BCS    LF5D1   ;2
       LDA    $D2,X   ;4
       STA    $D3,X   ;4
       LDA    $CA,X   ;4
       STA    $CB,X   ;4
       SEC            ;2
       SBC    #$06    ;2
       STA    $CA,X   ;4
       DEX            ;2
       BPL    LF5BA   ;2
LF5D1: INX            ;2
       BNE    LF597   ;2
LF5D4: LDY    #$FF    ;2
       STY    tempVariable2     ;3
       LDX    #$07    ;2
LF5DA: LDA    $D2,X   ;4
       CMP    #tempVariable    ;2
       BCC    LF5E2   ;2
       LDA    #$87    ;2
LF5E2: JSR    CalcXPos   ;6
       STA    $E0,X   ;4
       TYA            ;2
       ORA    $E0,X   ;4
       STA    $E0,X   ;4
       DEX            ;2
       BPL    LF5DA   ;2
       LDA    mapCombatState   ;3
       BMI    LF5F6   ;2
LF5F3: JMP    LF6BF   ;3
LF5F6: LDA    logoTimer     ;3
       BNE    LF5F3   ;2
       LDA    SWCHA   ;4
       LSR            ;2
       LSR            ;2
       LSR            ;2
       LSR            ;2
       EOR    #$0F    ;2
       LDX    #$05    ;2
LF605: LDY    LFC95,X ;4
       CPY    targetSector     ;3
       BNE    LF60E   ;2
       AND    #$0B    ;2
LF60E: LDY    LFC9B,X ;4
       CPY    targetSector     ;3
       BNE    LF617   ;2
       AND    #$07    ;2
LF617: DEX            ;2
       BPL    LF605   ;2
       TAX            ;2
       BNE    LF621   ;2
       STA    $9E     ;3
       BEQ    LF652   ;2
LF621: LDA    $9E     ;3
       BEQ    LF629   ;2
       DEC    $9E     ;5
       BPL    LF652   ;2
LF629: LDA    #$10    ;2
       STA    $9E     ;3
       LDA    targetSector     ;3
       TAY            ;2
       CLC            ;2
       ADC    LFD09,X ;4
       BMI    LF652   ;2
       CMP    #$24    ;2
       BCS    LF652   ;2
       STA    targetSector     ;3
       TYA            ;2
       JSR    RemoveShip   ;6
       LDA    targetSector     ;3
       CMP    shipSector     ;3
       BEQ    LF652   ;2
       JSR    ScanSector   ;6
       CLC            ;2
       ADC    #$05    ;2
       TAY            ;2
       LDA    targetSector     ;3
       JSR    LFB87   ;6
LF652: LDY    #$01    ;2
LF654: STX    tempVariable     ;3
       STA    tempVariable2     ;3
       LDA.wy $00C3,Y ;4
       LDX    #$00    ;2
LF65D: CMP    #$06    ;2
       BCC    LF667   ;2
       INX            ;2
       SEC            ;2
       SBC    #$06    ;2
       BPL    LF65D   ;2
LF667: DEY            ;2
       BPL    LF654   ;2
       SEC            ;2
       SBC    tempVariable2     ;3
       BPL    LF674   ;2
       EOR    #$FF    ;2
       CLC            ;2
       ADC    #$01    ;2
LF674: STA    tempVariable2     ;3
       TXA            ;2
       SEC            ;2
       SBC    tempVariable     ;3
       BPL    LF681   ;2
       EOR    #$FF    ;2
       CLC            ;2
       ADC    #$01    ;2
LF681: SED            ;2
       CLC            ;2
       ADC    tempVariable2     ;3
       STA    $BF     ;3
       LDY    damageWR     ;3
       CPY    #$B0    ;2
       BCC    LF690   ;2
       CLC            ;2
       ADC    $BF     ;3
LF690: CLD            ;2
       STA    $BF     ;3
       LDY    REFP1   ;3
       BMI    LF6BF   ;2
       LDY    targetSector     ;3
       CPY    shipSector     ;3
       BEQ    LF6BF   ;2
       JSR    ScanShipSector   ;6
       SEC            ;2
       SBC    #$05    ;2
       TAY            ;2
       JSR    LFB85   ;6
       LDA    targetSector     ;3
       STA    shipSector     ;3
       LDA    mapCombatState     ;3
       EOR    #$80    ;2
       STA    mapCombatState     ;3
       LDA    #$FE    ;2
       STA    enemy0Z     ;3
       LDA    #$C8    ;2
       STA    starFieldSpeed     ;3
       LDA    #$00    ;2
       STA    $9D     ;3
       STA    $A0     ;3
LF6BF: LDA    flashTimerRed     ;3
       CMP    #$01    ;2
       BNE    LF6DC   ;2
       LDA    randomNumber     ;3
       AND    #$7F    ;2
       STA    enemy0Y     ;3
       JSR    NewRandomValue   ;6
       LDA    $9D     ;3
       BNE    LF6D8   ;2
       LDA    randomNumber     ;3
       AND    #$7F    ;2
       STA    enemy0X     ;3
LF6D8: LDA    #$FC    ;2
       STA    enemy0Z     ;3
LF6DC: LDA    frameCounter     ;3
       AND    #$01    ;2
       ORA    logoTimer     ;3
       BNE    LF6F0   ;2
       LDX    #$05    ;2
LF6E6: LDA    $A1,X   ;4
       CLC            ;2
       ADC    enemy0Y,X   ;4
       STA    enemy0Y,X   ;4
       DEX            ;2
       BPL    LF6E6   ;2


    ORG $F6F0

LF6F0: LDX    #$01    ;2
LF6F2: LDA    enemy0Z,X   ;4
       CMP    #$B0    ;2
       BCC    LF6FA   ;2
       LDA    #$B0    ;2
LF6FA: LSR            ;2
       LSR            ;2
       LSR            ;2
       LSR            ;2
       TAY            ;2
       STX    tempVariable     ;3
       LDX    starFieldSpeed     ;3
       BNE    LF711   ;2
       JSR    ScanShipSector   ;6
       CMP    #$09    ;2
       BNE    LF711   ;2
       TYA            ;2
       CLC            ;2
       ADC    #$0C    ;2
       TAY            ;2
LF711: LDX    tempVariable     ;3
       CPX    #$01    ;2
       BNE    LF71C   ;2
       TYA            ;2
       CLC            ;2
       ADC    #$18    ;2
       TAY            ;2
LF71C: LDA    shapesize,Y ;4
       STA    vertShipSize,X   ;4
       LDA    enemyshapetab,Y ;4
       STA    enemyShipShape,X   ;4
       DEX            ;2
       BPL    LF6F2   ;2
       LDA    logoTimer     ;3
       BEQ    LF730   ;2
       JMP    LF8E1   ;3
LF730: LDA    frameCounter     ;3
       AND    #$3F    ;2
       BNE    LF73B   ;2
       LDA    #$01    ;2
       JSR    LFBBA   ;6
LF73B: LDA    $9D     ;3
       BNE    LF741   ;2
       STA    enemy1Y     ;3
LF741: JSR    ScanShipSector   ;6
       STA    tempVariable2     ;3
       LDA    starFieldSpeed     ;3
       BEQ    LF75D   ;2
       LDA    frameCounter     ;3
       AND    #$07    ;2
       BNE    LF755   ;2
       LDA    $BF     ;3
       JSR    LFBBA   ;6
LF755: LDA    #$FF    ;2
       STA    enemy0Z     ;3
       STA    $A5     ;3
       BNE    LF76B   ;2
LF75D: LDA    tempVariable2     ;3
       CMP    #$09    ;2
       BEQ    LF77C   ;2
       CMP    #$05    ;2
       BNE    LF76B   ;2
       LDA    #$00    ;2
       STA    enemy0Y     ;3
LF76B: LDA    flashTimerRed     ;3
       ORA    flashTimerBlue     ;3
       ORA    $9D     ;3
       BNE    LF7A8   ;2
       LDA    frameCounter     ;3
       AND    #$7F    ;2
       BNE    LF7A8   ;2
       JSR    LFBA6   ;6
LF77C: LDA    starFieldSpeed     ;3
       CMP    #$50    ;2
       BCS    LF792   ;2
       LDA    tempVariable2     ;3
       CMP    #$09    ;2
       BEQ    LF7A8   ;2
       CMP    #$05    ;2
       BEQ    LF7A8   ;2
       LDA    enemy0Z     ;3
       CMP    #$B1    ;2
       BCS    LF7A8   ;2
LF792: LDA    #$00    ;2
       STA    $A2     ;3
       LDX    level    ; X-> 00 v 01 v 02 v 03
       LDA    LFCAE,X       ; A-> FE v FD v FC v FB
       STA    $A6     ;3
       STA    $9D     ;3
       LDA    enemy0Z     ;3
       STA    enemy1Z     ;3
       LDY    enemy0Y     ;3
       INY            ;2
       STY    enemy1Y     ;3
LF7A8: LDY    #$02    ;2
       LDX    $9D     ;3
LF7AC: LDA    enemy1Y     ;3
       CPY    #$02    ;2
       BNE    LF7B4   ;2
       LDA    enemy0X     ;3
LF7B4: CMP    LFCC0,Y ;4
       BCS    LF7BB   ;2
       LDX    #$00    ;2
LF7BB: CMP    LFCC1,Y ;4
       BCC    LF7C2   ;2
       LDX    #$00    ;2
LF7C2: DEY            ;2
       DEY            ;2
       BPL    LF7AC   ;2
       STX    $9D     ;3
       LDA    $9D     ;3
       BEQ    LF812   ;2
       LDA    enemy1Z     ;3
       CMP    #$06    ;2
       BCS    LF816   ;2
       LDY    #$7F    ;2
       STY    flashTimerOrange     ;3
       JSR    LFBCB   ;6
       LDA    #$00    ;2
       STA    $9D     ;3
       LDA    damageLS     ;3
       AND    #$0F    ;2
       CMP    #$0B    ;2
       BNE    LF7E8   ;2
       JMP    LFBE5   ;3
LF7E8: LDA    randomNumber     ;3
       CMP    #$40    ;2
       BCS    LF812   ;2
       AND    #$03    ;2
       TAX            ;2
       LSR            ;2
       TAY            ;2
       LDA    LFD97,X ;4
       AND    LFD9C,X ;4
       STA    tempVariable     ;3
       EOR.wy $00B9,Y ;4
       AND    LFD9C,X ;4
       BEQ    LF812   ;2
       LDA.wy $00B9,Y ;4
       AND    LFD9B,X ;4
       ORA    tempVariable     ;3
       STA.wy $00B9,Y ;5
       LDA    #$60    ;2
       STA    $B4     ;3
LF812: LDA    #$FC    ;2
       STA    enemy1Z     ;3
LF816: LDY    #$04    ;2
LF818: LDA.wy $00AE,Y ;4
       CMP    LFCC0,Y ;4
       BCS    LF827   ;2
       LDX    LFCC6,Y ;4
       STX    $A1,Y   ;4
       BNE    LF831   ;2
LF827: CMP    LFCC1,Y ;4
       BCC    LF831   ;2
       LDX    LFCC7,Y ;4
       STX    $A1,Y   ;4
LF831: DEY            ;2
       DEY            ;2
       BPL    LF818   ;2
       LDA    VSYNC   ;3
       ORA    VBLANK  ;3
       AND    #$C0    ;2
       BNE    LF840   ;2
LF83D: JMP    LF8E1   ;3
LF840: STA    CXCLR   ;3
       LDA    enemy0X     ;3
       CMP    #$46    ;2
       BCC    LF83D   ;2
       CMP    #$5E    ;2
       BCS    LF83D   ;2
       LDA    $AA     ;3
       BEQ    LF875   ;2
       LDA    $9D     ;3
       BEQ    LF875   ;2
       LDA    enemy1Y     ;3
       CMP    #$45    ;2
       BCC    LF875   ;2
       SEC            ;2
       SBC    vertShotSize     ;3
       CMP    #$4C    ;2
       BCS    LF875   ;2
       LDA    #$F0    ;2
       STA    enemy1Z     ;3
       LDA    #$00    ;2
       STA    $A2     ;3
       STA    tempVariable3     ;3
       STA    $A6     ;3
       STA    $9D     ;3
       LDA    #$5F    ;2
       STA    flashTimerBlue     ;3
       BNE    LF83D   ;2
LF875: LDA    flashTimerRed     ;3
       ORA    starFieldSpeed     ;3
       BNE    LF8E1   ;2
       LDA    enemy0Y     ;3
       CMP    #$45    ;2
       BCC    LF8E1   ;2
       SEC            ;2
       SBC    vertShipSize     ;3
       CMP    #$4C    ;2
       BCS    LF8E1   ;2
       LDA    tempVariable2     ;3
       CMP    #$09    ;2
       BNE    LF8B5   ;2
       LDA    enemy0Z     ;3
       ORA    $A0     ;3
       CMP    #$0F    ;2
       BCS    LF8E1   ;2
       SED            ;2
       LDA    $9F     ;3
       CLC            ;2
       ADC    #$01    ;2
       STA    $9F     ;3
       CLD            ;2
       LDA    #$0F    ;2
       STA    $B4     ;3
       STA    enemy0Z     ;3
       LDA    #$99    ;2
       STA    $A0     ;3
       STA    bcdenergyHi     ;3
       STA    $BC     ;3
       LDA    #$AA    ;2
       STA    damageLS     ;3
       STA    damageWR     ;3
       BNE    LF8D9   ;2
LF8B5: LDA    $AA     ;3
       BEQ    LF8E1   ;2
       LDA    #$F0    ;2
       STA    enemy0Z     ;3
       LDA    #$7F    ;2
       STA    flashTimerRed     ;3
       JSR    ScanShipSector   ;6
       SEC            ;2
       SBC    #$01    ;2
       TAY            ;2
       JSR    LFB85   ;6
       SED                      ; Dezimal mode
       SEC                      ; 
       LDA    bcdEnemyCounter   ;
       SBC    #$01              ;
       STA    bcdEnemyCounter   ; One enemy ship zapped!
       CLD                      ; Binary mode
       BNE    LF8D9   ;2
       JMP    LFBE5   ;3
LF8D9: LDA    #$00    ;2
       STA    $A1     ;3
       STA    $A3     ;3
       STA    $A5     ;3
LF8E1: LDX    #$00    ;2
       LDA    enemy0X     ;3
       JSR    SetXPos   ;6
       INX            ;2
       LDA    enemy0X     ;3
       CLC            ;2
       ADC    #$08    ;2
       JSR    SetXPos   ;6
       INX            ;2
       LDA    #$A3    ;2
       JSR    SetXPos   ;6
       INX            ;2
       LDA    #$06    ;2
       JSR    SetXPos   ;6
       STA    WSYNC   ;3
       STA    HMOVE   ;3
       LDX    #$02    ;2
LF903: LDA    flashTimerBlue,X   ;4
       BEQ    LF909   ;2
       DEC    flashTimerBlue,X   ;6
LF909: DEX            ;2
       BPL    LF903   ;2
       LDA    $B4     ;3
       BEQ    LF91A   ;2
       DEC    $B4     ;5
       STA    AUDF0   ;3
       LDX    #$0C    ;2
       STX    AUDC0   ;3
       BNE    LF94D   ;2
LF91A: LDX    #$08    ;2
       STX    AUDC0   ;3
       LDA    $AA     ;3
       BEQ    LF92E   ;2
       LDA    randomNumber     ;3
       AND    #$03    ;2
       STA    AUDF0   ;3
       LDA    #$0F    ;2
       STA    AUDV0   ;3
       BNE    LF94F   ;2
LF92E: LDX    #$00    ;2
       LDA    logoTimer     ;3
       BNE    LF948   ;2
       LDX    #$08    ;2
       LDA    starFieldSpeed     ;3
       BNE    LF940   ;2
       LDA    #$FF    ;2
       LDX    #$03    ;2
       BNE    LF948   ;2
LF940: CMP    #$16    ;2
       BCS    LF948   ;2
       EOR    #$FF    ;2
       LDX    #$06    ;2
LF948: LSR            ;2
       LSR            ;2
       LSR            ;2
       STA    AUDF0   ;3
LF94D: STX    AUDV0   ;3
LF94F: LDA    $9D     ;3
       BEQ    LF965   ;2
       LDA    enemy1Z     ;3
       LSR            ;2
       LSR            ;2
       LSR            ;2
       STA    AUDF1   ;3
       LSR            ;2
       EOR    #$8F    ;2
       STA    AUDV1   ;3
       LDA    #$08    ;2
       STA    AUDC1   ;3
       BNE    LF9AE   ;2
LF965: LDA    flashTimerRed     ;3
       ORA    flashTimerBlue     ;3
       ORA    flashTimerOrange     ;3
       BEQ    LF97E   ;2
       LSR            ;2
       LSR            ;2
       LSR            ;2
       STA    AUDV1   ;3
       LDA    randomNumber     ;3
       ORA    #$18    ;2
       STA    AUDF1   ;3
       LDA    #$08    ;2
       STA    AUDC1   ;3
       BNE    LF9AE   ;2
LF97E: LDA    tuneIndex    ;3
       BEQ    LF9AC   ;2
       LDA    frameCounter     ;3
       AND    #$07    ;2
       BNE    LF9AE   ;2
       LDA    #$0C    ;2
       STA    AUDC1   ;3
       DEC    $B7     ;5
       LDA    $B7     ;3
       BEQ    LF9AC   ;2
       BPL    LF9AE   ;2
       DEC    tuneIndex     ;5
       LDX    tuneIndex     ;3
       LDA    LFC49,X ;4
       BNE    LF9A1   ;2
       STA    tuneIndex     ;3
       BEQ    LF9AC   ;2
LF9A1: STA    AUDF1   ;3
       LSR            ;2
       LSR            ;2
       LSR            ;2
       LSR            ;2
       LSR            ;2
       STA    $B7     ;3
       LDA    #$08    ;2
LF9AC: STA    AUDV1   ;3
LF9AE: STA    HMCLR   ; Clear horizontal movement registers
       LDA    #$10    ;
       STA    HMM0    ;
       LDA    #$F0    ;
       STA    HMM1    ; Center crosshair
       JMP    Mainloop; Goto Mainloop

; Draws one Mapline (7 Pixels high) 
; Player 0 starts at Pixel 37
; Player 1 starts at Pixel 85
; The spacing is 8 Bytes between sectors
; in: all 6 sprite pointers are already set

DrawMapLine:        LDY #$07            ; 1 Mapline = 7 Pixels high
NextPixelLine:      STA WSYNC           ; Finish current line
                    LDA (targetPtr),Y   ; Set shape 1 as...
                    STA GRP0            ; ... Player 0
                    LDA ($F3),Y         ; Set shape 4 as...
                    STA GRP1            ; ... Player 1
                    LDA ($F7),Y         ; Store shape 6 temporary...
                    STA tempVariable    ; ...for faster access later
                    LDA ($F1),Y         ; Store shape 3 in X...
                    TAX                 ; ...for faster access later
                    LDA ($EF),Y         ; Set shape 2 as...
                    STA GRP0            ; ... Player 0
                    NOP                 ; Wait
                    STX GRP0            ; Player 0 -> shape 3
                    LDX tempVariable    ; X -> shape 6
                    LDA ($F5),Y         ; Set shape 1 as...
                    STA GRP1            ;  ... Player 1
                    NOP                 ; Wait
                    STX GRP1            ; Player 0 -> Shape 6
                    DEY                 ; Done all 7 lines?
                    BNE NextPixelLine   ; N: Draw next line
                    STY GRP0            ; Y: Clear Sprites
                    STY GRP1            ;
                    RTS                 ; Return

; Draws the (eventually) animated Activision logo (8 Pixels high) 
; It later jumps into to the bigsprite routine
; in: all 6 sprite pointers are already set

DrawLogo:           LDY #$0F            ; Assume offset for 'Activision'
                    LDA #$07            ; 8 lines to draw
                    STA tempVariable3   ; store temporary
                    LDA logoTimer       ; A-> logotimer
                    LSR                 ; 
                    LSR                 ;
                    LSR                 ; divide by 8
                    CMP #$14            ; A < $14 ?
                    BCS Bigspriteready  ; Y: Draw 'Activision'
                    LDY #$07            ; N: ass. offset for 'Copyright'
                    CMP #$0C            ; A >= $0C ?
                    BCC Bigspriteready  ; Y: Draw 'Copyright'
                    SBC #$04            ; Offsets from $08 to $0F
                    TAY                 ; Y->A
Bigspriteready:     STY tempVariable2   ; tempVariable2->logo Offset
                    JMP BigSpriteLine   ; Draw the logo

; Draws one stateline (7 Pixels high) 
; The spacing is 0 Pixels between chars
; For an in-depth analysis consult:
; http://www.biglist.com/lists/stella/archives/199704/msg00137.html
; in: all 6 sprite pointers are already set
                                                            
DrawStateLine:      LDA #$06            ; 7 lines to draw
                    STA tempVariable2   ; Shape offset synchronized...
                    STA tempVariable3   ; to linenmuber 
BigSpriteLine:      STA WSYNC           ; Finish current line...
                    LDA #$01            ; (0) 
                    STA VDELP0          ; (2) USE GRPXA with...
                    STA VDELP1          ; (5) ...both players.
                    LDX #$0A            ; (8) Burn some cycles...
Wait2:              DEX                 ; (10)
                    BPL Wait2           ; (12)
                    LDA VSYNC           ; (64)another three cycles
LFA18:              LDY tempVariable2   ; (67)Y-> current shape offset
                    LDA ($F7),Y         ; (70)Store shape 6 in...
                    STA $0188           ; (75)...tempVariable /4 Cycles!
                    LDA ($F5),Y         ; ( 3)Store shape 5 in... 
                    TAX                 ; ( 8)...X
                    LDA (targetPtr),Y   ; (10)Set shape 1 as...
                    STA GRP0            ; (15)... player 0
                    LDA ($EF),Y         ; (18)Set shape 2 as...
                    STA GRP1            ; (23)... player 1
                    LDA ($F1),Y         ; (26)Set shape 3 as... 
                    STA GRP0            ; (31)... player 0
                    LDA ($F3),Y         ; (34)A-> shape 4
                    LDY tempVariable    ; (39)Y-> shape 6
                    STA GRP1            ; (42)player 1 -> shape 4
                    STX GRP0            ; (45)player 0 -> shape 5 
                    STY GRP1            ; (48)player 1 -> shape 6
                    STA GRP0            ; (51)Move GRP1 -> GRP1A
                    DEC tempVariable2   ; (54)decrease shape offset
                    DEC tempVariable3   ; (59)all lines done?
                    BPL LFA18           ; (64)N: Draw next line
                    LDA #$00            ; Y: Clear all four sprite...
                    STA GRP0            ; ... register: GRP0, GRP1,...
                    STA GRP1            ; ... GRP0A, GRP1A
                    STA GRP0            ;
                    STA GRP1            ;
                    STA VDELP0          ; Restore sprite register...
                    STA VDELP1          ; ...to use GRP0 and GRP1
                    RTS                 ; return

; Removes the ship in desired sector
; in:  A & Y -> Sector number

RemoveShip:         CMP shipSector      ; Ship in sector?
                    BEQ Return          ; Y: Don't remove it!
                    CLC                 ; N: Clear carry
                    ROR                 ; Carry: 0 -> Low v 1 -> High
                    TAX                 ; X-> sectorPair number
                    LDA sectorList,X    ; Load sectorpair
                    BCC LowNibble2      ; If carry 0 -> Select LowNibble
                    LSR                 ; Else select HiNibble
                    LSR                 ;
                    LSR                 ;
                    LSR                 ;
LowNibble2:         AND #$0F            ; A-> Nibble of sector
                    SEC                 ;
                    SBC #ID_SHIP        ; Remove ship in sector
                    STA tempVariable    ; Store new nibble temporary
                    TYA                 ; A-> Sector number
                    CLC                 ; Clear carry
                    ROR                 ; Carry: 0 -> Low v 1 -> High
                    TAX                 ; X-> sectorPair number
                    LDA sectorList,X    ; Load sectorpair
                    BCC LowNibble3      ; If carry 0 -> Select LowNibble
                    AND #$0F            ; Clear current HiNibble...
                    ASL tempVariable    ;
                    ASL tempVariable    ;
                    ASL tempVariable    ;
                    ASL tempVariable    ;
                    ORA tempVariable    ; ... and OR new sector in
                    JMP WriteSector     ; 
LowNibble3:         AND #$F0            ; Clear current LowNibble...
                    ORA tempVariable    ; ... and OR new sector in
WriteSector:        STA sectorList,X    ; write sector, now without ship
Return:             RTS                 ; Return

; GameInit sets everything to a defined state when reset|select pressed

GameInit:           LDY #$00            ;
                    STY NUSIZ0          ; no missile / player copies
                    STY NUSIZ1          ; missile / player small sized
ClearMem:           STY $00,X           ;
                    INX                 ;
                    CPX #$B8            ; Clear 0 - B7 v 85 - B7
                    BNE ClearMem        ;       (Reset / New game)

                    LDX #$28            ;
CopyMem:            LDA inittab,X       ;
                    STA levelDelay,X    ;
                    DEX                 ; Copy inittab-FD0A to
                    BPL CopyMem         ;     levelDelay-E0

                    LDY level           ; Y-> 00 v 01 v 02 v 03
                    LDA bcdmaxenemies,Y ; A-> 09 v 17 v 23 v 31
                    STA bcdEnemyCounter ; Init bcdEnemyCounter

                    LDA levelptrtab,Y   ; A-> 'E' v 'L' v 'W' v 'S'
                    STA LevelSprite     ; Init LevelSprite

                    LDX #$11            ; 18 bytes for 36 Map sectors
InitMap:            LDA sectorinittab,X ;
                    STA sectorList,X    ; copy 18 sectors directly
                    DEX                 ;
                    LDA sectorinittab,X ;
                    ORA level           ; The other 18 sectors have 0-4
                    STA sectorList,X    ; enemies ORed in
                    DEX                 ;
                    BPL InitMap         ; Init all 36 Sectors of the Map

                    LDA SWCHB           ; Read console switches
                    LSR                 ; 
                    AND #$64            ; A-> .DifP0DifP1. .ColBW..
                    LDY logoTimer       ; Y-> 0 (Reset) v !=0 (Select)
                    BEQ DisplayCombat   ; Reset?
                    ORA #$80            ; N: Show Map screen
DisplayCombat:      STA mapCombatState  ; Y: Show combat screen
                    RTS                 ; Return

; Generates a new (pseudo-)random number

NewRandomValue:     LDA randomNumber    ; A-> current random value
                    BNE SeedNonZero     ; Is it zero?
                    LDA #$FF            ; Y: Newly Init it with $FF
SeedNonZero:        ASL                 ; N: generate a new random...
                    ASL                 ; value out of the current...
                    ASL                 ; by wild shifting its Bits in
                    EOR randomNumber    ; and out :-)
                    ASL                 ; (The common Activision...
                    ROL randomNumber    ; routine for that)
                    RTS                 ; return


; The horizontal positioning routine has two entry points: 
; CalcXPos: Just calculates a position 
; SetXPos:  Immediately positiones a player sprite
; For more information on that matter please consult:
; http://www.biglist.com/lists/stella/archives/199709/msg00006.html
; in:  A -> Desired Horizontal position + 9
; in:  tempVariable2 -> 00 (for set mode) v FF (for calculate mode)
; in:  X -> Player number to be set (00 v 01)
; out: A -> Offset value for fine positioning
; out: Y -> Cycle Delay v 00 when position is already set
; out: X -> remains unaltered

CalcXPos:           CLC                 ;
                    ADC #$F0            ; Position += F0
                    JMP CalcPos         ;
SetXPos:            LDY #$00            ; 
                    CLC                 ; Begin at leftmost edge
                    ADC #$25            ; ? Strange, 9 Pixels shifted ?
                    STY tempVariable2   ; Init tempVariable2 with zero
CalcPos:            TAY                 ; Y -> Position
                    AND #$0F            ; 
                    STA tempVariable    ; store away rest of Position/16
                    TYA                 ; A -> Position
                    LSR                 ; Right shift 4 times to...
                    LSR                 ; ...divide by 16. Since need...
                    LSR                 ; ... to divide by 15 we...
                    LSR                 ; ... correct that below!
                    TAY                 ; Y -> Position/16
                    CLC                 ; 
                    ADC tempVariable    ; Add the rest of the division
                    CMP #$0F            ; Rest is bigger than 15?
                    BCC NoCorrect       ; N: Goto NoCorrect
                    SBC #$0F            ; Y: Subtract another 15
                    INY                 ; Add another cycle delay
NoCorrect:          EOR #$07            ; Translate fine positioning...
                    ASL                 ; ...from values from 0-15...
                    ASL                 ; ...into offsets from -7 to +8
                    ASL                 ; 
                    ASL                 ; Move offset to upper nibble
                    BIT tempVariable2   ; SetPosition?
                    BPL ExecuteSetXPos  ; Y: Set Position
                    RTS                 ; N: Return

ExecuteSetXPos:     STA HMP0,X          ; Store fine positioning value
                    STA WSYNC           ; Sync to video
Wait:               DEY                 ; Have 5 cycles delay per delay
                    BPL Wait            ; (Which is 15 pixel!)
                    STA RESP0,X         ; Hit register for positioning
                    RTS                 ; Return

; Prepare everything for drawing the stateslines

PrepareStates:      NOP                 ; Burn two cycles
                    LDA #$A0            ; Fine position player 0...
                    STA HMP0            ; ... 6 pixels right
                    LDA #$B0            ; Fine position player 1...
                    STA HMP1            ; ... 7 pixels right
                    STA RESP0           ; Hit register for positioning
                    STA RESP1           ; both players at 8 pixels space
                    STA WSYNC           ; Finish current line
                    STA HMOVE           ; Execute fine positioning
                    JSR ScanShipSector  ; Scan ship sector
                    LDY #$D4            ; Assume we're alone (green)
                    CMP #$05            ; Clear Sector?
                    BEQ LFB2F           ; Y: Green Background
                    LDY #$84            ; N: Assume base near (blue)
                    CMP #$09            ; Base Near?
                    BEQ LFB2F           ; Y: Blue Background
                    LDY #$44            ; N: Red Background
LFB2F:              TYA                 ; A -> Background color
                    EOR saverColor      ; ...change it...
                    AND brightnessMask  ; ...when in saver mode
                    TAX                 ; X -> Background color
                    LDY #$80            ; Assume playfield dark blue
                    STA HMCLR           ; Clear h-mot registers
                    STA WSYNC           ; Finish current line
                    STA HMOVE           ; ????? Useless instruction
                    LDA frameCounter    ; A -> frame counter
                    AND #$1F            ; Mask lower 5 bits
                    CMP #$0F            ; A < 0F?
                    BCS NoBlink         ; Y: NoBlink
                    LDA bcdenergyHi     ; A -> bcdenergyHi
                    CMP #$10            ; Energy higher than 1000 units?
                    BCS NoBlink         ; N: NoBlink
                    LDY #$26            ; Blink playfield orange
NoBlink:            TYA                 ; A -> Playfield color
                    EOR saverColor      ; ...change it...
                    AND brightnessMask  ; ...when in saver mode
                    STA WSYNC           ; Finish current line
                    STX COLUBK          ; Background green v blue v red
                    STA COLUPF          ; Playfield dark blue v orange
                    LDX #$01            ; 
                    STX CTRLPF          ; Reflected playfield
                    DEX                 ; 
                    STX GRP0            ; Clear both sprites
                    STX GRP1            ;
                    DEX                 ; Cause playfield to draw a... 
                    STX PF2             ; block around states: PF2 -> FF
                    LDA #$03            ; players 3 copies/close spacing
                    STA NUSIZ0          ;
                    STA NUSIZ1          ;
                    LDX #$0A            ; Set 6 MSBs of sprite pointers
                    LDA #$FF            ; ... to FF in Map mode
InitSpritePtrs2:    STA $EE,X           ; Write MSB
                    DEX                 ; Skip to next pointer
                    DEX                 ; Done?
                    BPL InitSpritePtrs2 ; N: Continue
                    RTS                 ; Y: Return

; Scan/ship/sector returns the current statenibble 
; of the desired sector in A
; in:  A -> Number of sector to scan
; out: A -> Nibble holding the state of scanned sector

ScanShipSector:     LDA shipSector      ; A-> sector to get
ScanSector:         CLC                 ; Clear carry
                    ROR                 ; Carry: 0 -> Low v 1 -> High
                    TAX                 ; X-> sectorPair number
                    LDA sectorList,X    ; Load sectorpair
                    BCC LowNibble       ; If carry 0 -> Select LowNibble
                    LSR                 ; Else select HiNibble
                    LSR                 ;
                    LSR                 ;
                    LSR                 ;
LowNibble:          AND #$0F            ; A holds now the nibble that
                    RTS                 ; represents the desired sector

LFB85: LDA    shipSector     ;3
LFB87: STY    tempVariable     ;3
       CLC            ;2
       ROR            ;2
       TAX            ;2
       LDA    sectorList,X   ;4
       BCC    LFB9F   ;2
       AND    #$0F    ;2
       ASL    tempVariable     ;5
       ASL    tempVariable     ;5
       ASL    tempVariable     ;5
       ASL    tempVariable     ;5
       ORA    tempVariable     ;3
       JMP    LFBA3   ;3
LFB9F: AND    #$F0    ;2
       ORA    tempVariable     ;3
LFBA3: STA    sectorList,X   ;4
       RTS            ;6

LFBA6: LDY    #$04    ;2
LFBA8: JSR    NewRandomValue   ;6
       LDA    randomNumber     ;3
       AND    #$07    ;2
       TAX            ;2
       LDA    LFD30,X ;4
       STA.wy $00A1,Y ;5
       DEY            ;2
       BPL    LFBA8   ;2
       RTS            ;6

LFBBA: STA    tempVariable     ;3
       SED            ;2
       SEC            ;2
       LDA    $BC     ;3
       SBC    tempVariable     ;3
       STA    $BC     ;3
       LDA    bcdenergyHi     ;3
       SBC    #$00    ;2
       JMP    LFBD4   ;3
LFBCB: STA    tempVariable     ;3
       LDA    bcdenergyHi     ;3
       SEC            ;2
       SED            ;2
       SBC    tempVariable     ;3
       CLD            ;2
LFBD4: BCS    LFBE1   ;2
       LDA    #$00    ;2
       STA    $BC     ;3
       STA    bcdenergyHi     ;3
       PLA            ;4
       PLA            ;4
       JMP    LFBE5   ;3
LFBE1: STA    bcdenergyHi     ;3
       CLD            ;2
       RTS            ;6

LFBE5: LDY    #$0C    ;2
       SED            ;2
       SEC            ;2
       LDX    level        ;
       LDA    bcdmaxenemies,X   ;
       SBC    bcdEnemyCounter   ;
       CMP    bcdmaxenemies,X   ;
       BNE    LFBFB   ;2
       LDY    #$12    ;2
       CLC            ;2
       ADC    LFCB2,X ;4
LFBFB: STY    tuneIndex     ;3
       CLC            ;2
       ADC    $C9     ;3
       ADC    $C9     ;3
       ADC    $C9     ;3
       ADC    $C9     ;3
       ADC    $C9     ;3
       SEC            ;2
       SBC    #$10    ;2
       SEC            ;2
       SBC    $9F     ;3
       BCS    LFC12   ;2
       LDA    #$00    ;2
LFC12: STA    $C1     ;3
       LDA    #$00    ;2
       SEC            ;2
       SBC    $BE     ;3
       STA    $C2     ;3
       DEC    $86     ;5
       LDA    $C1     ;3
       SBC    $BD     ;3
       BCS    LFC25   ;2
       LDA    #$00    ;2
LFC25: STA    $C1     ;3
       LDA    #$00    ;2
       STA    $9D     ;3
       STA    $AC     ;3
       STA    $AD     ;3
       STA    enemy0Y     ;3
       STA    enemy1Y     ;3
       STA    $AA     ;3
       CLD            ;2
       JMP    LF8E1   ;3

       ORG $FC39

LFC39: .byte $0A,$05,$02

bcdmaxenemies: .byte $09,$17,$23,$31

LFC40: .byte $00,$05,$0A,$0F,$00,$00,$05,$0A,$0F
LFC49: .byte $00,$FE,$3F,$7E,$3E,$7A,$3A,$79,$FE,$3E,$7E,$FE,$00,$EC,$2B,$EE
       .byte $73,$FD,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
LFC67: .byte $0A,$19,$01,$1D
LFC6B: .byte $03,$04,$05,$09,$0B,$0F,$10,$11,$12,$19,$14,$18,$1A,$1E,$1F,$20
       .byte $01,$01,$01,$00,$02,$06,$07,$08,$16,$17,$1D,$1D,$1C,$22,$23,$1D
LFC8B: .byte $02,$04,$00,$02,$02,$04,$00,$02,$02,$04
LFC95: .byte $00,$06,$0C,$12,$18,$1E
LFC9B: .byte $05,$0B,$11,$17,$1D,$23

backcoltab: .byte $0F ; WHITE     (Stars)
            .byte $00 ; BLACK     (Background)
            .byte $84 ; BLUE      (Background)
            .byte $44 ; RED       (Background)
            .byte $28 ; ORANGE    (Background)

stateptrtab: .byte $B8 ; offset D(amage)
             .byte $bf ; offset E(nergy)
             .byte $D4 ; offset S(hield)
             .byte $DB ; offset W(arp)


levelptrtab: .byte $BF      ; offset E(nsign)
             .byte $C6      ; offset L(eader)
             .byte $DB      ; offset W(ing Commander)
             .byte $D4      ; offset S(tarmaster)

LFCAE: .byte $FE,$FD,$FC,$FB
LFCB2: .byte $11,$23,$37,$49
LFCB6: .byte $FB,$FC,$FD,$FE,$FF,$01,$02,$03,$04,$05
LFCC0: .byte $0A
LFCC1: .byte $86,$0B,$9E,$04,$B0
LFCC6: .byte $01
LFCC7: .byte $FF,$01,$FF,$01,$FF

mapshapeptrtab: .byte $4A,$54,$5E,$E1,$EF,$4F,$59,$63,$E8,$F5

logoptrtab: .byte $A0,$FD,$B0,$FD,$C0,$FD,$D0,$FD,$E0,$FD,$F0,$FD

inittab: .byte $20                        ; level delay
         .byte $AA, $AA                   ; damage
         .byte $99, $99                   ; energy
         .byte $00, $00                   ; stardate
         .byte $00, $00                   ; warp energy
         .byte $AA, $AA                   ; score
         .byte $0B                        ; ship sector
         .byte $0B                        ; new ship sector
         .byte $FF, $FF, $FF, $FF         ; starbase-shields
         .byte $06                        ; number of starbases + 2
         .byte $0B
         .byte $19
         .byte $37
         .byte $47
         .byte $55
         .byte $65
         .byte $73
         .byte $83
         .byte $14
         .byte $82
         .byte $32
         .byte $6E
         .byte $4B
         .byte $78
         .byte $41
         .byte $1E
         .byte $00
         .byte $03
         .byte $07
         .byte $04
         .byte $24
         ; Next two bytes are used too!

LFD09: .byte $00,$06,$FA,$00,$FF,$05,$F9,$00,$01,$07,$FB

sectorinittab: .byte $42, $00, $00
               .byte $00, $01, $54
               .byte $00, $00, $00
               .byte $00, $10, $00
               .byte $40, $00, $40
               .byte $00, $32
               ; Next byte is used too!
          
LFD25: .byte  0,  1, -1,  0,  0,  1, -1,  0,  0,  1, -1
LFD30: .byte -1, -1, -1, -1,  0,  1,  1,  1

enemyshapetab: .byte $00
               .byte $11
               .byte $20
               .byte $2D
               .byte $38
               .byte $4A
               .byte $41
               .byte $41
               .byte $41
               .byte $41
               .byte $41
               .byte $42

               .byte $03
               .byte $1B
               .byte $30
               .byte $3D
       .byte $48,$5B,$4A,$41,$41,$41,$41,$41,$89,$A7,$C1,$D7,$7A,$6E,$64,$5B
       .byte $52,$4A,$41,$42

shapesize: .byte $0B,$09,$07,$05,$03,$02,$01,$01,$01,$01,$01,$01
           .byte $0B,$09,$07,$05,$03,$03,$02,$01,$01,$01,$01,$01
           .byte $18,$14,$10,$0C,$08,$06,$04,$03,$03,$02,$01,$01
       
LFD80: .byte $FA,$FA,$FA,$FA,$FA,$FC,$FC,$FC,$FE,$FE,$FE,$02,$02,$02,$04,$04
       .byte $04,$06,$06,$06,$08,$08,$08
LFD97: .byte $0B,$C0,$0D,$E0
LFD9B: .byte $F0
LFD9C: .byte $0F,$F0,$0F,$F0

; Copyright 1982 Activision FDA0-FDFF:
       .byte $00 ;      
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $F7 ; xxxx xxx
       .byte $95 ; x  x x x
       .byte $87 ; x    xxx
       .byte $80 ; x       
       .byte $90 ; x  x    
       .byte $F0 ; xxxx    
       .byte $AD ; x x xx x
       .byte $A9 ; x x x  x
       .byte $E9 ; xxx x  x
       .byte $A9 ; x x x  x
       .byte $ED ; xxx xx x
       .byte $41 ;  x     x
       .byte $0F ;     xxxx
       .byte $00 ;         
       .byte $47 ;  x   xxx
       .byte $41 ;  x     x
       .byte $77 ;  xxx xxx
       .byte $55 ;  x x x x
       .byte $75 ;  xxx x x
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $50 ;  x x    
       .byte $58 ;  x xx   
       .byte $5C ;  x xxx  
       .byte $56 ;  x x xx 
       .byte $53 ;  x x  xx
       .byte $11 ;    x   x
       .byte $F0 ; xxxx    
       .byte $00 ;         
       .byte $03 ;       xx
       .byte $00 ;         
       .byte $4B ;  x  x xx
       .byte $4A ;  x  x x 
       .byte $6B ;  xx x xx
       .byte $00 ;         
       .byte $08 ;     x   
       .byte $00 ;         
       .byte $BA ; x xxx x 
       .byte $8A ; x   x x 
       .byte $BA ; x xxx x 
       .byte $A2 ; x x   x 
       .byte $3A ;   xxx x 
       .byte $80 ; x       
       .byte $FE ; xxxxxxx 
       .byte $00 ;         
       .byte $80 ; x       
       .byte $80 ; x       
       .byte $AA ; x x x x 
       .byte $AA ; x x x x 
       .byte $BA ; x xxx x 
       .byte $22 ;   x   x 
       .byte $27 ;   x  xxx
       .byte $02 ;       x 
       .byte $E9 ; xxx x  x
       .byte $AB ; x x x xx
       .byte $AF ; x x xxxx
       .byte $AD ; x x xx x
       .byte $E9 ; xxx x  x
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $11 ;    x   x
       .byte $11 ;    x   x
       .byte $17 ;    x xxx
       .byte $15 ;    x x x
       .byte $17 ;    x xxx
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $77 ;  xxx xxx
       .byte $54 ;  x x x  
       .byte $77 ;  xxx xxx
       .byte $51 ;  x x   x
       .byte $77 ;  xxx xxx
       .byte $00 ;         

; enemy shapes
       .byte $03 ;       XX| $FE00
       .byte $07 ;      XXX| $FE01
       .byte $07 ;      XXX| $FE02
       .byte $0F ;     XXXX| $FE03
       .byte $0D ;     XX X| $FE04
       .byte $D8 ; XX XX   | $FE05
       .byte $F0 ; XXXX    | $FE06
       .byte $E0 ; XXX     | $FE07
       .byte $E0 ; XXX     | $FE08
       .byte $60 ;  XX     | $FE09
       .byte $20 ;   X     | $FE0A
       .byte $00 ;         | $FE0B
       .byte $00 ;         | $FE0C
       .byte $00 ;         | $FE0D
       .byte $00 ;         | $FE0E
       .byte $00 ;         | $FE0F
       .byte $00 ;         | $FE10

       .byte $01 ;        X| $FE11
       .byte $03 ;       XX| $FE12
       .byte $07 ;      XXX| $FE13
       .byte $07 ;      XXX| $FE14
       .byte $6D ;  XX XX X| $FE15
       .byte $78 ;  XXXX   | $FE16
       .byte $70 ;  XXX    | $FE17
       .byte $30 ;   XX    | $FE18
       .byte $10 ;    X    | $FE19
       .byte $00 ;         | $FE1A
       .byte $00 ;         | $FE1B
       .byte $00 ;         | $FE1C
       .byte $00 ;         | $FE1D
       .byte $00 ;         | $FE1E
       .byte $00 ;         | $FE1F

       .byte $01 ;        X| $FE20
       .byte $03 ;       XX| $FE21
       .byte $07 ;      XXX| $FE22
       .byte $35 ;   XX X X| $FE23
       .byte $38 ;   XXX   | $FE24
       .byte $18 ;    XX   | $FE25
       .byte $08 ;     X   | $FE26
       .byte $00 ;         | $FE27
       .byte $00 ;         | $FE28
       .byte $00 ;         | $FE29
       .byte $00 ;         | $FE2A
       .byte $00 ;         | $FE2B
       .byte $00 ;         | $FE2C

       .byte $01 ;        X| $FE2D
       .byte $03 ;       XX| $FE2E
       .byte $0D ;     XX X| $FE2F
       .byte $0C ;     XX  | $FE30
       .byte $04 ;      X  | $FE31
       .byte $00 ;         | $FE32
       .byte $00 ;         | $FE33
       .byte $00 ;         | $FE34
       .byte $00 ;         | $FE35
       .byte $00 ;         | $FE36
       .byte $00 ;         | $FE37

       .byte $01 ;        X| $FE38
       .byte $05 ;      X X| $FE39
       .byte $02 ;       X | $FE3A
       .byte $00 ;         | $FE3B
       .byte $00 ;         | $FE3C
       .byte $00 ;         | $FE3D
       .byte $00 ;         | $FE3E
       .byte $00 ;         | $FE3F
       .byte $00 ;         | $FE40

       .byte $01 ;        X| $FE41
       .byte $00 ;         | $FE42
       .byte $00 ;         | $FE43
       .byte $00 ;         | $FE44
       .byte $00 ;         | $FE45
       .byte $00 ;         | $FE46
       .byte $00 ;         | $FE47
       .byte $00 ;         | $FE48
       .byte $00 ;         | $FE49

       .byte $01 ;        X| $FE4A
       .byte $01 ;        X| $FE4B
       .byte $00 ;         | $FE4C
       .byte $00 ;         | $FE4D
       .byte $00 ;         | $FE4E
       .byte $00 ;         | $FE4F
       .byte $00 ;         | $FE50
       .byte $00 ;         | $FE51
       .byte $01 ;        X| $FE52
       .byte $01 ;        X| $FE53
       .byte $01 ;        X| $FE54
       .byte $00 ;         | $FE55
       .byte $00 ;         | $FE56
       .byte $00 ;         | $FE57
       .byte $00 ;         | $FE58
       .byte $00 ;         | $FE59
       .byte $00 ;         | $FE5A
       .byte $01 ;        X| $FE5B
       .byte $03 ;       XX| $FE5C
       .byte $01 ;        X| $FE5D
       .byte $00 ;         | $FE5E
       .byte $00 ;         | $FE5F
       .byte $00 ;         | $FE60
       .byte $00 ;         | $FE61
       .byte $00 ;         | $FE62
       .byte $00 ;         | $FE63
       .byte $01 ;        X| $FE64
       .byte $03 ;       XX| $FE65
       .byte $03 ;       XX| $FE66
       .byte $01 ;        X| $FE67
       .byte $00 ;         | $FE68
       .byte $00 ;         | $FE69
       .byte $00 ;         | $FE6A
       .byte $00 ;         | $FE6B
       .byte $00 ;         | $FE6C
       .byte $00 ;         | $FE6D
       .byte $01 ;        X| $FE6E
       .byte $03 ;       XX| $FE6F
       .byte $07 ;      XXX| $FE70
       .byte $07 ;      XXX| $FE71
       .byte $03 ;       XX| $FE72
       .byte $01 ;        X| $FE73
       .byte $00 ;         | $FE74
       .byte $00 ;         | $FE75
       .byte $00 ;         | $FE76
       .byte $00 ;         | $FE77
       .byte $00 ;         | $FE78
       .byte $00 ;         | $FE79
       .byte $01 ;        X| $FE7A
       .byte $03 ;       XX| $FE7B
       .byte $07 ;      XXX| $FE7C
       .byte $0F ;     XXXX| $FE7D
       .byte $0F ;     XXXX| $FE7E
       .byte $07 ;      XXX| $FE7F
       .byte $03 ;       XX| $FE80
       .byte $01 ;        X| $FE81
       .byte $00 ;         | $FE82
       .byte $00 ;         | $FE83
       .byte $00 ;         | $FE84
       .byte $00 ;         | $FE85
       .byte $00 ;         | $FE86
       .byte $00 ;         | $FE87
       .byte $00 ;         | $FE88
       .byte $03 ;       XX| $FE89
       .byte $0F ;     XXXX| $FE8A
       .byte $1F ;    XXXXX| $FE8B
       .byte $3F ;   XXXXXX| $FE8C
       .byte $3F ;   XXXXXX| $FE8D
       .byte $3F ;   XXXXXX| $FE8E
       .byte $7F ;  XXXXXXX| $FE8F
       .byte $7F ;  XXXXXXX| $FE90
       .byte $7F ;  XXXXXXX| $FE91
       .byte $7F ;  XXXXXXX| $FE92
       .byte $FF ; XXXXXXXX| $FE93
       .byte $FF ; XXXXXXXX| $FE94
       .byte $FF ; XXXXXXXX| $FE95
       .byte $FF ; XXXXXXXX| $FE96
       .byte $7F ;  XXXXXXX| $FE97
       .byte $7F ;  XXXXXXX| $FE98
       .byte $7F ;  XXXXXXX| $FE99
       .byte $7F ;  XXXXXXX| $FE9A
       .byte $3F ;   XXXXXX| $FE9B
       .byte $3F ;   XXXXXX| $FE9C
       .byte $3F ;   XXXXXX| $FE9D
       .byte $1F ;    XXXXX| $FE9E
       .byte $0F ;     XXXX| $FE9F
       .byte $03 ;       XX| $FEA0
       .byte $00 ;         | $FEA1
       .byte $00 ;         | $FEA2
       .byte $00 ;         | $FEA3
       .byte $00 ;         | $FEA4
       .byte $00 ;         | $FEA5
       .byte $00 ;         | $FEA6
       .byte $03 ;       XX| $FEA7
       .byte $0F ;     XXXX| $FEA8
       .byte $1F ;    XXXXX| $FEA9
       .byte $1F ;    XXXXX| $FEAA
       .byte $1F ;    XXXXX| $FEAB
       .byte $3F ;   XXXXXX| $FEAC
       .byte $3F ;   XXXXXX| $FEAD
       .byte $3F ;   XXXXXX| $FEAE
       .byte $7F ;  XXXXXXX| $FEAF
       .byte $7F ;  XXXXXXX| $FEB0
       .byte $7F ;  XXXXXXX| $FEB1
       .byte $7F ;  XXXXXXX| $FEB2
       .byte $3F ;   XXXXXX| $FEB3
       .byte $3F ;   XXXXXX| $FEB4
       .byte $3F ;   XXXXXX| $FEB5
       .byte $1F ;    XXXXX| $FEB6
       .byte $1F ;    XXXXX| $FEB7
       .byte $1F ;    XXXXX| $FEB8
       .byte $0F ;     XXXX| $FEB9
       .byte $03 ;       XX| $FEBA
       .byte $00 ;         | $FEBB
       .byte $00 ;         | $FEBC
       .byte $00 ;         | $FEBD
       .byte $00 ;         | $FEBE
       .byte $00 ;         | $FEBF
       .byte $00 ;         | $FEC0
       .byte $03 ;       XX| $FEC1
       .byte $0F ;     XXXX| $FEC2
       .byte $0F ;     XXXX| $FEC3
       .byte $0F ;     XXXX| $FEC4
       .byte $1F ;    XXXXX| $FEC5
       .byte $1F ;    XXXXX| $FEC6
       .byte $1F ;    XXXXX| $FEC7
       .byte $3F ;   XXXXXX| $FEC8
       .byte $3F ;   XXXXXX| $FEC9
       .byte $1F ;    XXXXX| $FECA
       .byte $1F ;    XXXXX| $FECB
       .byte $1F ;    XXXXX| $FECC
       .byte $0F ;     XXXX| $FECD
       .byte $0F ;     XXXX| $FECE
       .byte $0F ;     XXXX| $FECF
       .byte $03 ;       XX| $FED0
       .byte $00 ;         | $FED1
       .byte $00 ;         | $FED2
       .byte $00 ;         | $FED3
       .byte $00 ;         | $FED4
       .byte $00 ;         | $FED5
       .byte $00 ;         | $FED6
       .byte $01 ;        X| $FED7
       .byte $07 ;      XXX| $FED8
       .byte $07 ;      XXX| $FED9
       .byte $0F ;     XXXX| $FEDA
       .byte $0F ;     XXXX| $FEDB
       .byte $1F ;    XXXXX| $FEDC
       .byte $1F ;    XXXXX| $FEDD
       .byte $0F ;     XXXX| $FEDE
       .byte $0F ;     XXXX| $FEDF
       .byte $07 ;      XXX| $FEE0
       .byte $07 ;      XXX| $FEE1
       .byte $01 ;        X| $FEE2
       .byte $00 ;         | $FEE3
       .byte $00 ;         | $FEE4

LFEE5: .byte $00,$00,$00,$00,$03,$03,$03,$00,$FD,$FD,$FD

stateshapeptr: .byte $6B ; -> '0'
               .byte $72 ; -> '1'
               .byte $79 ; -> '2'
               .byte $80 ; -> '3'
               .byte $87 ; -> '4'
               .byte $8E ; -> '5'
               .byte $95 ; -> '6'
               .byte $9C ; -> '7'
               .byte $A3 ; -> '8'
               .byte $AA ; -> '9'
               .byte $4B ; -> ' '
               .byte $D4 ; -> 'S'
               .byte $C6 ; -> 'L'
               .byte $CD ; -> 'R'
               .byte $DB ; -> 'W'
               .byte $BF ; -> 'E'

LFF00: .byte $00,$10,$20,$01,$13,$13,$17,$7F,$EB,$7F,$17,$13,$13,$01,$00,$00
LFF10: .byte $00,$00,$00,$00,$01,$01,$01,$00,$FF,$FF,$FF,$01,$13,$17,$3F,$6B
       .byte $3F,$17,$13,$01,$00,$00

noradartab: .byte $00,$00,$00,$00,$04,$05,$05,$05,$05,$09

       .byte $01,$0B,$1F,$35,$1F,$0B
       .byte $01,$00,$00,$00,$00,$00,$00,$09,$0F,$1B,$0F,$09,$00,$00,$00,$00
       .byte $00,$00,$05,$0F,$05
       
       ; Mapsprite '0 enemies + no ship' (Starts at FF4F)
       ; + empty score shape (Starts at FF4B)
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         

       ; Mapsprite '0 enemies + ship' (Starts at FF54)
       .byte $00 ;         
       .byte $00 ;         
       .byte $08 ;     x   
       .byte $1C ;    xxx  
       .byte $08 ;     x   

       ; Mapsprite '1 enemy + no ship' (Starts at FF59)
       .byte $00 ;         
       .byte $00 ;         
       .byte $40 ;  x      
       .byte $40 ;  x      
       .byte $00 ;         

       ; Mapsprite '1 enemy + ship' (Starts at FF5E)
       .byte $00 ;         
       .byte $00 ;         
       .byte $48 ;  x  x   
       .byte $5C ;  x xxx  
       .byte $08 ;     x   

       ; Mapsprite '2 enemies + no ship' (Starts at FF63)

       .byte $00 ;         
       .byte $00 ;        
       .byte $41 ;  x     x
       .byte $41 ;  x     x
       .byte $00 ;         

       ; Mapsprite '2 enemies + ship' (Starts at FF68)
       .byte $00 ;         
       .byte $00 ;         
       .byte $49 ;  x  x  x
       .byte $5D ;  x xxx x
       .byte $08 ;     x   

       .byte $00 ;         
       .byte $00 ;         

       ; Charsprite '0' (Starts at FF6B)
       .byte $3C ; ..xxxx..
       .byte $66 ; .xx..xx.
       .byte $66 ; .xx..xx.
       .byte $66 ; .xx..xx.
       .byte $66 ; .xx..xx.
       .byte $66 ; .xx..xx.
       .byte $3C ; ..xxxx..

       ; Charsprite '1' (Starts at FF72)
       .byte $3C ; ..xxxx..
       .byte $18 ; ...xx...
       .byte $18 ; ...xx...
       .byte $18 ; ...xx...
       .byte $18 ; ...xx...
       .byte $38 ; ..xxx...
       .byte $18 ; ...xx...

       ; Charsprite '2' (Starts at FF79)
       .byte $7E ; .xxxxxx.
       .byte $60 ; .xx.....
       .byte $60 ; .xx.....
       .byte $3C ; ..xxxx..
       .byte $06 ; .....xx.
       .byte $46 ; .x...xx.
       .byte $3C ; ..xxxx..

       ; Charsprite '3' (Starts at FF80)
       .byte $3C ; ..xxxx..
       .byte $46 ; .x...xx.
       .byte $06 ; .....xx.
       .byte $0C ; ....xx..
       .byte $06 ; .....xx.
       .byte $46 ; .x...xx.
       .byte $3C ; ..xxxx..

       ; Charsprite '4' (Starts at FF87)
       .byte $0C ; ....xx..
       .byte $0C ; ....xx..
       .byte $7E ; .xxxxxx.
       .byte $4C ; .x..xx..
       .byte $2C ; ..x.xx..
       .byte $1C ; ...xxx..
       .byte $0C ; ....xx..

       ; Charsprite '6' (Starts at FF8E)
       .byte $7C ; .xxxxx..
       .byte $46 ; .x...xx.
       .byte $06 ; .....xx.
       .byte $7C ; .xxxxx..
       .byte $60 ; .xx.....
       .byte $60 ; .xx.....
       .byte $7E ; .xxxxxx.

       ; Charsprite '6' (Starts at FF95)
       .byte $3C ; ..xxxx..
       .byte $66 ; .xx..xx.
       .byte $66 ; .xx..xx.
       .byte $7C ; .xxxxx..
       .byte $60 ; .xx.....
       .byte $62 ; .xx...x.
       .byte $3C ; ..xxxx..

       ; Charsprite '7' (Starts at FF9C)
       .byte $18 ; ...xx...
       .byte $18 ; ...xx...
       .byte $18 ; ...xx...
       .byte $0C ; ....xx..
       .byte $06 ; .....xx.
       .byte $42 ; .x....x.
       .byte $7E ; .xxxxxx.
       
       ; Charsprite '8' (Starts at FFA3)
       .byte $3C ; ..xxxx..
       .byte $66 ; .xx..xx.
       .byte $66 ; .xx..xx.
       .byte $3C ; ..xxxx..
       .byte $66 ; .xx..xx.
       .byte $66 ; .xx..xx.
       .byte $3C ; ..xxxx..
       
       ; Charsprite '9' (Starts at FFAA)
       .byte $3C ; ..xxxx..
       .byte $46 ; .x...xx.
       .byte $06 ; .....xx.
       .byte $3E ; ..xxxxx.
       .byte $66 ; .xx..xx.
       .byte $66 ; .xx..xx.
       .byte $3C ; ..xxxx..
       
       ; Charsprite ':' (Starts at FFB1)
       .byte $00 ; ........
       .byte $18 ; ...xx...
       .byte $00 ; ........
       .byte $00 ; ........
       .byte $00 ; ........
       .byte $18 ; ...xx...
       .byte $00 ; ........
       
       ; Charsprite 'D' (Starts at FFB8)
       .byte $7C ; .xxxxx..
       .byte $66 ; .xx..xx.
       .byte $66 ; .xx..xx.
       .byte $66 ; .xx..xx.
       .byte $66 ; .xx..xx.
       .byte $66 ; .xx..xx.
       .byte $7C ; .xxxxx..

       ; Charsprite 'E' (Starts at FFBF)
       .byte $7E ; .xxxxxx.
       .byte $60 ; .xx.....
       .byte $60 ; .xx.....
       .byte $78 ; .xxxx...
       .byte $60 ; .xx.....
       .byte $60 ; .xx.....
       .byte $7E ; .xxxxxx.

       ; Charsprite 'L' (Starts at FFC6)
       .byte $7E ; .xxxxxx.
       .byte $60 ; .xx.....
       .byte $60 ; .xx.....
       .byte $60 ; .xx.....
       .byte $60 ; .xx.....
       .byte $60 ; .xx.....
       .byte $60 ; .xx.....

       ; Charsprite 'R' (Starts at FFCD)
       .byte $66 ; .xx..xx.
       .byte $6C ; .xx.xx..
       .byte $68 ; .xx.x...
       .byte $7C ; .xxxxx..
       .byte $66 ; .xx..xx.
       .byte $66 ; .xx..xx.
       .byte $7C ; .xxxxx..

       ; Charsprite 'S' (Starts at FFD4)
       .byte $3C ; ..xxxx..
       .byte $46 ; .x...xx.
       .byte $06 ; .....xx.
       .byte $3C ; ..xxxx..
       .byte $60 ; .xx.....
       .byte $62 ; .xx...x.
       .byte $3C ; ..xxxx..

       ; Charsprite 'W' (Starts at FFDB)
       .byte $63 ; .xx...xx
       .byte $77 ; .xxx.xxx
       .byte $7F ; .xxxxxxx
       .byte $6B ; .xx.x.xx
       .byte $6B ; .xx.x.xx
       .byte $63 ; .xx...xx
       .byte $63 ; .xx...xx

       ; Mapsprite '3 enemies + no ship' (Starts at FFE8)
       .byte $41 ;  x     x
       .byte $41 ;  x     x
       .byte $00 ;         
       .byte $00 ;         
       .byte $00 ;         
       .byte $40 ;  x      
       .byte $40 ;  x      

       ; Mapsprite '3 enemies + ship' (Starts at FFEF)
       .byte $41 ;  x     x
       .byte $41 ;  x     x
       .byte $08 ;     x   
       .byte $1C ;    xxx  
       .byte $08 ;     x   
       .byte $40 ;  x      
       .byte $40 ;  x      

       ; Mapsprite 'base + ship' (Starts at FFF5)
       .byte $00 ;         
       .byte $08 ;     x   
       .byte $5D ;  x xxx x
       .byte $7F ;  xxxxxxx
       .byte $5D ;  x xxx x
       .byte $08 ;     x   
       
       ; Mapsprite 'base + no ship' (Starts at FFFB)
       .byte $00 ;         
       .byte $08 ;     x   
       .byte $55 ;  x x x x
       .byte $63 ;  xx   xx
       .byte $55 ;  x x x x
       .byte $08 ;     x   

       .word Start
       .word 0
